//***********************************************************************
//
// RC_tank_with_PWM.c
//
// Author:	Jeremy Greenwood
//
// Description:
//
//			Bluetooth remote control treaded chassis firmware using an
//		ATtiny2313 AVR microcontroller. The program receives PWM and
//		direction commands on the UART via a Bluetooth module to control
//		two bidirectional motors.
//
//***********************************************************************

#define F_CPU 8000000UL											// 8 MHz
#define USART_BAUDRATE 9600										// Define baud rate

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avriomacros.h>
#include "uart2313.h"

// Motor pin Defines
#define MA1		PB0		// right motor
#define MA2		PB1
#define MB1		PB3		// left motor
#define MB2		PB4

#define ENA		PB2		// OC0A
#define ENB		PD5		// OC0B

// Motor Control Macros
#define	MOV_FWD() 	{on(MB2,B);off(MB1,B);on(MA2,B);off(MA1,B);}
#define	MOV_BCK() 	{off(MB2,B);on(MB1,B);off(MA2,B);on(MA1,B);}
#define	STOP()	 	{off(MB2,B);off(MB1,B);off(MA2,B);off(MA1,B);}

// Macro to initilize timer1
#define init_timer()	{TCCR1B |= (1 << WGM12) | (1 << CS10) | (1 << CS11); OCR1A = 125; TIMSK |= (1 << OCIE1A);}

// Prototypes
void init_PWM( void );

volatile int	rx_buffer[3],									// indexes: 0 -> direction, 1 -> left motor PWM, 2 -> right motor PWM
				rx_index = 0,
				read = 0,
				rx_complete = 0,								// Declare flag for when a complete instruction is recieved
				ms_tic = 0;

// 1ms timer interrupt
ISR( TIMER1_COMPA_vect ) {									

	ms_tic++;			// incremet 1ms counter
}

// Bluetooth serial communication interrupt
ISR( USART_RX_vect ) {

	int input = Rx();											// Recieve byte

	if( input == '{' )
		rx_index = 0;											// Reset buffer when { is recieved (beginning of instruction)
	else {
		if( input == '}' ) {									// Finds the end of an instruction when } is recieved
			rx_complete = 1;									// Flag set to indicate instruction is completely recieved	
			ms_tic = 0;											// Reset millisecond counter (timeout)
		}
		else
			if( rx_index < sizeof(rx_buffer) )
				rx_buffer[ rx_index++ ] = input;
	}
}

int main (void) {

	// Define outputs for motor logic pins
	out(MA1,B);
	out(MA2,B);
	out(MB1,B);
	out(MB2,B);
	out(ENA,B);
	out(ENB,D);

	init_UART(F_CPU,USART_BAUDRATE);
	en_Rx();
	init_PWM();
	init_timer();

	STOP();

   	while(1) {
		
		if( rx_complete ) {

			// determine direction
			switch( rx_buffer[0] ) {

				case '0':
					MOV_BCK();
					break;
				case '1':
					MOV_FWD();
					break;
				default:
					STOP();
			}
				
			// set left motor PWM
			OCR0B = (int) rx_buffer[1] + 155;
			if( OCR0B < 160 || OCR0B > 255 )
				OCR0B = 0;

			// set right motor PWM
			OCR0A = (int) rx_buffer[2] + 155;
			if( OCR0A < 160 || OCR0A > 255 )
				OCR0A = 0;

			// silence the motor noise (PWM of zero has a spike)
			if( OCR0A == 0 && OCR0B == 0 )
				STOP();

			rx_complete = 0;
		}

		if( ms_tic >= 100 ) {  		// stop moving if nothing is received for 100 milliseconds
		
			STOP();
			ms_tic = 0;				// reset 1ms counter
		}		
	}
}

void init_PWM( void ) {

//	**** Set up PWM output on OC0B and OC0A *******	

  	TCCR0A = (1 << WGM01) | (1 << WGM00)				// Configure timer mode to "Fast PWM" mode
		| (1 << COM0A1) | (0 << COM0A0)					// Clear OC0A on Compare Match, set OC0A at TOP
		| (1 << COM0B1) | (0 << COM0B0);				// Clear OC0B on Compare Match, set OC0B at TOP

  	TCCR0B = (0 << CS02) | (1 << CS01) | (0 << CS00)	// Set clock select bit (table 41)
		| (0 << WGM02);									// Set to PWM mode 3 (table 40)
}
