#########################################################################
##
##  RC_tank_with_PWM.py
##
##  Author: Jeremy Greenwood
##
##  Description:
##
##          Serves as an accelerometer GUI viewer and sends data packets
##      via Bluetooth to control direction and PWM of two motors on a
##      treaded chassis vehicle. Python script for Series 60 Symbian
##      smartphone, designed for use with Nokia E75 (3rd Edition, Feature
##      Pack 2) using PyS60 2.0.0
##
#########################################################################

import sensor
import btsocket
import e32
import time
import audio    # text to speech
import appuifw
import sys
import graphics

sensor_acc = None
data_x = data_y = data_z = 0

# Colors
RGB_BLACK = (0, 0, 0)
RGB_WHITE = (255, 255, 255)
RGB_RED = (255, 0, 0)
RGB_GREEN = (0, 255, 0)
RGB_BLUE = (0, 0, 255)

# Accelerometer limits and PWM parameters
z_min = 8
z_max = 50      # determine max speed
y_max = 40      # determine steering
percent_min = 20
percent_max = 100
steer_sensitivity = (percent_max - percent_min) / y_max

canvas = None
img = None
f_busy = True

def transmit_data():
    global s, data_x, data_y, data_z
    
    if data_z > 0:
        direction = '1'
    else:
        direction = '0'

    # normalize data_z to PWM duty cycle
    m = 100 / (z_max - z_min)
    b = -m * z_min
    top_speed = m * abs(data_z) + b
    
    if top_speed < 0:
        top_speed = 0
    if top_speed > 100:
        top_speed = 100

    percent = 100 - steer_sensitivity * abs(data_y)
    if percent < 20:
        percent = 20
    if percent > 100:
        percent = 100

    # reduce slow motor to a percentage of top_speed
    if data_y > 0:
        left_PWM = top_speed
        right_PWM = percent * top_speed / 100
    else:
        right_PWM = top_speed
        left_PWM = percent * top_speed / 100

    s.send('{')
    s.send(direction)
    s.send(chr(left_PWM))
    s.send(chr(right_PWM))
    s.send('}')

def format_number( val ):
    if len(str(val)) == 3:
        return str(val)
    if len(str(val)) == 2:
        if val >= 0:
            return ' ' + str(val)
        else:
            return '-  ' + str(val)[1]
    if len(str(val)) == 1:
        if val >= 0:
            return '   ' + str(val)

def cb_raw():
    global data_x, data_y, data_z
    data_x = sensor_acc.x
    data_y = sensor_acc.y
    data_z = sensor_acc.z
    cb_redraw()

def cb_redraw(dummy=(0, 0, 0, 0)):
    ''' Overwrite default screen redraw event handler '''
    global f_busy
    if not f_busy:
        e32.reset_inactivity()                  # keep backlight on
        img.clear(RGB_BLACK)       
        img.text((20, 110), u'%s' % (format_number(data_x)), fill=RGB_RED, font=(u'Series 60 Sans',50))
        img.text((120, 110), u'%s' % (format_number(data_y)), fill=RGB_GREEN, font=(u'Series 60 Sans',50))
        img.text((220, 110), u'%s' % (format_number(data_z)), fill=RGB_BLUE, font=(u'Series 60 Sans',50))
        canvas.blit(img)
    f_busy = False

def cb_quit():
    sensor_acc.stop_listening()
    global terminate
    terminate = True

appuifw.app.orientation = 'landscape'
appuifw.app.screen = 'large'
appuifw.app.exit_key_handler = cb_quit
canvas = appuifw.Canvas(redraw_callback = cb_redraw)
appuifw.app.body = canvas
img = graphics.Image.new(canvas.size)
appuifw.app.title=u"Bluetooth Remote Control"

channels = sensor.list_channels()
sensors = {}
for d in channels:
    sensors[d['name']] = 1

if sensors.has_key('AccelerometerXYZAxisData'):
    sensor_acc = sensor.AccelerometerXYZAxisData()
    sensor_acc.set_callback(data_callback=cb_raw)
    sensor_acc.start_listening()

terminate = False
f_busy = False

target_address = '00:06:66:04:0f:7d'
port = 1
s = btsocket.socket( btsocket.AF_BT, btsocket.SOCK_STREAM )

try:
    s.connect( (target_address, port) )
    connected = True
except:
    connected = False
    e32.start_exe(u'BtToggleApp.exe',u'True',0)
    print "Reattempting to connect...\n"
    s = btsocket.socket( btsocket.AF_BT, btsocket.SOCK_STREAM )
    try:
        time.sleep(2)   # Wait for Bluetooth to get up and running
        s.connect( (target_address, port) )
        print 'connected'
        connected = True
    except:
        print 'attempt failed'
                
if connected:
    audio.say(u"start your engines")
    state = "stop"
    
    terminate = False
    while not terminate:
        e32.ao_sleep(0.02)
        transmit_data()   
    
    s.send('5')
    s.close()
    audio.say(u"Mission complete.")
    print "Exiting..."

